set home [file dirname [info script]]
set exehome [pwd]
set libdir [file join $home lib]

lappend auto_path $libdir

#package require -exact XOTcl 1.6.7
package require -exact XOTcl 2.0b3

package require activerecord

source [file join db adaptors sqlite.tcl]
source [file join db associations.tcl]

namespace eval ActiveRecord {
	Class UserClassMethods -superclass ExtendedMetaclass
	Class UserInstanceMethods -superclass InstanceMethods
	Class TestClassMethods -superclass ExtendedMetaclass
	Class TestInstanceMethods -superclass InstanceMethods
	Class CommentClassMethods -superclass ExtendedMetaclass
	Class CommentInstanceMethods -superclass InstanceMethods
	
	Base User
	Base Test
	Base Comment
}

namespace eval TestFindBySql {
	proc test {} {
		puts [info level 0]
		array set sql [list \
			"select"    "id,fullname,department_id" \
			"from"      "users" \
			"where"     "department_id=1" \
			"order by"  "id" \
			"limit"     "10" \
		]
		
		set records [ActiveRecord::User find_by_sql sql]
		ActiveRecord::User inspect
		ActiveRecord::User destroy_objects $records
	}
}

namespace eval TestFindBy {
	proc test {} {
		puts [info level 0]
		set user [ActiveRecord::User find_by_id_and_passwd 6 "pass6"]
		ActiveRecord::User inspect
		$user destroy
	}
}

namespace eval TestFindAllBy {
	proc test {} {
		puts [info level 0]
		set records [ActiveRecord::User find_all_by_department_id 1]
		ActiveRecord::User inspect
		ActiveRecord::User destroy_objects $records
	}
}

namespace eval TestCreateAndUpdate {
	proc test {} {
		puts [info level 0]
		set record [::ActiveRecord::Test new]
		
		$record set property(name) "John"
		$record save
		ActiveRecord::Test inspect
		
		$record set property(name) "John Doe"
		$record save
		ActiveRecord::Test inspect
		
		$record destroy
	}
}

namespace eval TestAssociationMethod {
	proc test {} {
		puts [info level 0]
		set user [ActiveRecord::User find_by_id 6]
		set user_comments [$user comments]
		$user inspect
		ActiveRecord::Comment inspect
		ActiveRecord::Comment destroy_objects $user_comments
		$user destroy
	}
}

TestFindBySql::test
TestFindBy::test
TestFindAllBy::test
TestCreateAndUpdate::test
TestAssociationMethod::test
