namespace eval Inflector {
	variable dictionary
	array set dictionary {
		plurals {}
		singulars {}
		uncountables {}
	}
	proc plural {rule nocase replacement} {
		variable dictionary
		
		set dictionary(plurals) [linsert $dictionary(plurals) 0 [list $rule $nocase $replacement]]
	}
	
	proc singular {rule nocase replacement} {
		variable dictionary
		
		set dictionary(singulars) [linsert $dictionary(singulars) 0 [list $rule $nocase $replacement]]
	}
	
	proc irregular {singular plural} {
		plural "([string index $singular 0])[string range $singular 1 end]$" 1 "\\1[string range $plural 1 end]"
		singular "([string index $plural 0])[string range $plural 1 end]$" 1 "\\1[string range $singular 1 end]"
	}
	
	proc uncountable {words} {
		variable dictionary
		
		foreach word $words {
			lappend dictionary(uncountables) $word
		}
	}
	
	proc clear {scope all} {
		if {$scope == "all"} {
			foreach name [array names dictionary] {
				set dictionary(name) {}
			}
		} else {
			set dictionary($scope) {}
		}
	}
	
	proc pluralize {word} {
		variable dictionary
		
		set index [lsearch $dictionary(uncountables) $word]
		if {$index >= 0} {
			return $word
		} else {
			foreach exp $dictionary(plurals) {
				foreach {rule nocase replacement} $exp {
					if $nocase {set nocase -nocase} {set nocase --}
				}
				if [regsub $nocase $rule $word $replacement res] {break}
			}
			return $res
		}
	}
	
	proc singularize {word} {
		variable dictionary
		
		set index [lsearch $dictionary(uncountables) $word]
		if {$index >= 0} {
			return $word
		} else {
			foreach exp $dictionary(singulars) {
				foreach {rule nocase replacement} $exp {
					if $nocase {set nocase -nocase} {set nocase --}
				}
				if [regsub $nocase $rule $word $replacement res] {break}
			}
			return $res
		}
	}
	
	proc humanize {lower_case_and_underscored_word} {
		set str $lower_case_and_underscored_word
		if {$str == ""} {return ""}
		regsub -all -- {_id$} $str {} str
		regsub -all -- {_} $str { } str
		string totitle $str
	}
	
	proc camelize {lower_case_and_underscored_word {first_letter_in_uppercase true}} {
		set str $lower_case_and_underscored_word
		if {$str == ""} {return ""}
		if {$first_letter_in_uppercase == true} {
			set units {}
			foreach unit [split $str {/}] {
				set words {}
				foreach word [split $unit {_}] {
					lappend words [string totitle $word]
				}
				lappend units [join $words {}]
			}
			return [join $units {::}]
		} else {
			return "[string index $str 0][string range [camelize $str] 1 end]"
		}
	}
	
	proc titleize {word} {
		set str $word
		if {$str == ""} {return ""}
		set titlized_words {}
		foreach word [humanize [underscore $str]] {
			lappend titlized_words [string totitle $word]
		}
		return $titlized_words
	}
	
	proc underscore {camel_cased_word} {
		set str $camel_cased_word
		if {$str == ""} {return ""}
		regsub -all -- {::} $str {/} str
		regsub -all -- {([A-Z]+)([A-Z][a-z])} $str {\1_\2} str
		regsub -all -- {([a-z\d])([A-Z])} $str {\1_\2} str
		regsub -all -- {-} $str {_} str
		return [string tolower $str]
	}
	
	proc dasherize {underscored_word} {
		set str $underscored_word
		if {$str == ""} {return ""}
		regsub -all -- {_} $str {-} str
		return $str
	}
	
	proc demodulize {class_name_in_module} {
		set str $class_name_in_module
		if {$str == ""} {return ""}
		regsub -all {^.*::} $str {} str
		return $str
	}
	
	proc tableize {class_name} {
		set str $class_name
		if {$str == ""} {return ""}
		set str [namespace tail $str]
		pluralize [underscore $str]
	}
	
	proc classify {table_name} {
		# strip out any leading schema name
		set str $table_name
		if {$str == ""} {return ""}
		regsub {.*\.} $str "" str
		
		return [camelize [singularize $str]]
	}
	
	proc foreign_key {class_name {separate_class_name_and_id_with_underscore true}} {
		if {$separate_class_name_and_id_with_underscore == true} {
			set tail "_id"
		} else {
			set tail "id"
		}
	    return "[underscore [demodulize $class_name]]$tail"
	}
}

package require inflections

package provide inflector 0.1
